#include <vector>
#include <algorithm>
#include "Image.h"

Image::Image() {
    npx = npy = npix = 0;
    data = 0;
}

Image::Image(double* array, int n, int m) {
    data = new double[n*m];
    for (int j = 0; j < n; j++) {
	for (int i = 0; i < m; i++) {
	    data[i+j*m] = array[i+j*m];
	} 
    }
    npx = m;
    npy = n;
    npix = n * m;
}

Image::Image(const Image &img) {
    npx = img.getNpx();
    npy = img.getNpy();
    npix = npx * npy;
    data = new double[npix];
    for (int j = 0; j < npy; j++) {
	for (int i = 0; i < npx; i++) {
	    data[i+j*npx] = img.getValue(i, j);
	}
    }
}

Image::Image(int npx, int npy) {
    this->data = new double[npx*npy];
    this->npx = npx;
    this->npy = npy;
    this->npix = npx * npy;
    for (int i = 0; i < npix; i++) {
	data[i] = 0.0;
    }
}

Image::~Image() {
    delete [] data;
}

int Image::getNpx() const {
    return npx;
}

int Image::getNpy() const {
    return npy;
}

int Image::getNpix() const {
    return npix;
}

void Image::getNaxis(int *npx, int *npy) {
    *npx = this->npx;
    *npy = this->npy;
}

double Image::getValue(int x, int y) const {
    return data[x+npx*y];
}

void Image::setValue(int x, int y, double v) {
    data[x+npx*y] = v;
}

void Image::getData(float *oarray, int n) {
    for (int j = 0; j < npx; j++) {
	for (int i = 0; i < npy; i++) {
	    oarray[j+i*npx] = this->data[j+i*npx];
	} 
    }
}

Image& Image::operator=(const Image &img) {
    npx = img.getNpx();
    npy = img.getNpy();
    npix = npx * npy;
    data = new double[npix];
    for (int j = 0; j < npy; j++) {
	for (int i = 0; i < npx; i++) {
	    data[i+j*npx] = img.getValue(i, j);
	}
    }
}

Image& Image::operator/=(double a) {
    for (int i = 0; i < npix; i++) {
	this->data[i] /= a;
    }
    return *this;
}

double Image::median() {
    std::vector<double> v;
    for (int i = 0; i < npix; i++) {
	v.push_back(data[i]);
    }
    std::sort(v.begin(), v.end());
    return (v[npix/2]);
}

Image Image::getMedian(std::vector<Image> &v) {
    int npx = v[0].getNpx();
    int npy = v[0].getNpy();
    Image out(npx, npy);
    std::vector<double> a;
    for (int j = 0; j < npy; j++) {
	for (int i = 0; i < npx; i++) {
	    for (int k = 0; k < v.size(); k++) {
		a.push_back(v[k].getValue(i, j));
	    }
	    std::sort(a.begin(), a.end());
	    double med = a[a.size()/2];
	    out.setValue(i, j, med);
	    a.clear();
	}
    }
    return out;
}
