
#ifndef __IMAGE_H__
#define __IMAGE_H__

#include <vector>
#include "Point.h"

class Image {
    int npx;
    int npy;
    int npix;
    double *data;
 public:
    Image();			/* default constructor */
    Image(double* array, int n, int m);	/* constructor */
    Image(const Image &img);	/* copy constructor */
    Image(int npx, int npy);
    ~Image();			/* destructor */

    int getNpx() const;
    int getNpy() const;
    int getNpix() const;
    void getNaxis(int *npx, int *npy);

    double getValue(int x, int y) const;
    void setValue(int x, int y, double v);

    void getData(float *oarray, int n);

    Image crop(int xmin, int ymin, int npx, int npy);

    Image& operator=(const Image& img);

    Image& operator+=(double a);
    Image& operator+=(const Image& a);
    Image operator+(double a);
    Image operator+(const Image& a);

    Image& operator-=(double a);
    Image& operator-=(const Image& a);
    Image operator-(double a);
    Image operator-(const Image& a);

    Image& operator*=(double a);
    Image& operator*=(const Image& a);
    Image operator*(double a);
    Image operator*(const Image& a);

    Image& operator/=(double a);
    Image& operator/=(const Image& a);
    Image operator/(double a);
    Image operator/(const Image& a);

    double mean();
    double std();
    double median();

    Image boxavg(int width);
    Image convolve(const Image& a);

    std::vector<Point> findPeaks(double thres);

    static Image getMedian(std::vector<Image> &v);

    static Image gaussianImage(double sigma);
};
#endif
