import pyfits
import ds9
import math
import Image

def getArray(img):
    array = img.getData(img.getNpix())
    array.shape = (img.getNpy(), img.getNpx())
    return array

def display(img, d=None, frame=None):
    if (d == None):
        ds9_targets = ds9.ds9_targets()
        if  ds9_targets == None:
            d = ds9.ds9()
        else:
            d = ds9.ds9(ds9_targets[0].split()[1])
    array = getArray(img)
    if (frame != None):
        d.set("frame %d" % (frame))
    cmd = "array [ydim=%d,xdim=%d,bitpix=-32]" % (array.shape)
    d.set(cmd, array)
    return d

def readFits(name):
    hdulist = pyfits.open(name)
    img = Image.Image(hdulist[0].data)
    hdulist.close()
    return img

def writeFits(name, img):
    array = getArray(img)
    hdu = pyfits.PrimaryHDU(array)
    hdulist = pyfits.HDUList([hdu])
    hdulist.writeto(name, clobber=True)

def markPeaks(peaks, d=None):
    if (d == None):
        ds9_targets = ds9.ds9_targets()
        if  ds9_targets == None:
            d = ds9.ds9()
        else:
            d = ds9.ds9(ds9_targets[0].split()[1])
    for p in peaks:
        d.set("regions command {circle %d %d %f}" % (p.getX()+1, p.getY()+1, 3.0*math.log(p.getVal())))

img = readFits("../data/SUPA00207569.fits")
display(img, frame=1)

flat = readFits("flat.fits")

img /= flat
sky = img.median()
img -= sky
display(img, frame=2)

gauss = Image.Image_gaussianImage(1.0)
smth = img.convolve(gauss)
d = display(smth, frame=3)

peaks = smth.findPeaks(10000)
markPeaks(peaks, d=d)
